@@ -1,8 +1,10 @@
 #' Compute Value-at-Risk (VaR)
 #'
-#' @description \code{VaR} computes the Value-at-Risk of the distribution specified by the
-#'     arguments. The meaning of the parameters is the same as in \code{\link{ES}}, including
-#'     the recycling rules.
+#' @description
+#'
+#'     \code{VaR} is a generic function for computation of Value-at-Risk. The default method
+#'     computes VaR corresponding to a distribution, usually fitted or implied by a fitted
+#'     model.  There are also methods for data (numeric or matrix).
 #'
 #' @note We use the traditional definition of VaR as the negated lower quantile. For example,
 #'     if \eqn{X} are returns on an asset, VAR\eqn{{}_\alpha}{_a} = \eqn{-q_\alpha}{-q_a},
@@ -17,11 +19,13 @@
 #' 
 #'
 #' @details
-#'     \code{VaR} is S3 generic. The meaning of the parameters for its default method is the
-#'     same as in \code{\link{ES}}, including the recycling rules.
+#'
+#'     \code{VaR} is S3 generic. The meaning of the parameters for the methods defined in
+#'     \pkg{cvar} and described here are the same as in \code{\link{ES}}, including the
+#'     recycling rules.
 #'
 #'     \code{VaR_qf} and \code{VaR_cdf} are streamlined, non-generic, variants for the common
-#'     case when the \code{"..."} parameters are scalar. The parameters \code{x},
+#'     case when the \code{"..."} parameters are scalar. The parameters \code{p_loss},
 #'     \code{intercept}, and \code{slope} can be vectors, as for \code{VaR}.
 #'
 #'     Argument \code{dist} can also be a numeric vector. In that case the ES is computed,
@@ -31,6 +35,9 @@
 #'     passed eventually to \code{quantile()} and can be used, for example, to select a
 #'     non-defult method for the computation of quantiles.
 #'
+#'     If \code{dist} is of class \code{"matrix"}, then the numeric method is applied to each
+#'     column.
+#' 
 #'     In practice, we may need to compute VaR associated with data. The distribution comes
 #'     from fitting a model. In the simplest case, we fit a distribution to the data,
 #'     assuming that the sample is i.i.d. For example, a normal distribution \eqn{N(\mu,
@@ -40,6 +47,9 @@
 #'     if not, general optimisation routines can be used. More soffisticated models may be
 #'     used, even time series models such as GARCH and mixture autoregressive models.
 #'     
+#'     When argument \code{dist} represents log-returns, VaR is for the log-returns. Use
+#'     \code{transf = TRUE} to return its value for the returns.
+#'
 #'     
 #' @param tol tollerance
 #'
@@ -47,6 +57,8 @@
 #'
 #'     \code{\link[=predict.garch1c1]{predict}} for examples with fitted models
 #' 
+#' @concept Value at Risk
+#'
 #' @examples
 #' cvar::VaR(qnorm, c(0.01, 0.05), dist.type = "qf")
 #'
@@ -174,7 +186,8 @@ VaR_cdf <- function(dist, p_loss = 0.05, ...,
 #'
 #' @export
 VaR.default <- function(dist, p_loss = 0.05, dist.type = "qf", ...,
-                        intercept = 0, slope  = 1, tol = .Machine$double.eps^0.5, x){
+                        intercept = 0, slope  = 1, tol = .Machine$double.eps^0.5,
+                        transf = FALSE, x){
     if(!missing(x)){
         p_loss <- x
         warn_about_x("VaR")
@@ -200,13 +213,18 @@ VaR.default <- function(dist, p_loss = 0.05, dist.type = "qf", ...,
                   },
                   stop('argument dist.type should be one of "qf", "cdf" or "pdf"')
                   )
-    - intercept + slope * res
+    res <- - intercept + slope * res
+
+    if(transf)
+        - (exp(-res) - 1)
+    else
+        res
 }
 
 #' @name VaR
 #'
 #' @export
-VaR.numeric <- function(dist, p_loss = 0.05, ..., intercept = 0, slope  = 1, x){
+VaR.numeric <- function(dist, p_loss = 0.05, ..., intercept = 0, slope  = 1, transf = FALSE, x){
     if(!missing(x)){
         p_loss <- x
         warn_about_x("VaR")
@@ -215,59 +233,51 @@ VaR.numeric <- function(dist, p_loss = 0.05, ..., intercept = 0, slope  = 1, x){
     ## dist is raw data here
     res <- - quantile(dist, p_loss, ...)
 
-    - intercept + slope * res
+    res <- - intercept + slope * res
+    if(transf)
+        - (exp(-res) - 1)
+    else
+        res
+}
+
+#' @name VaR
+#'
+#' @export
+VaR.matrix <- function(dist, p_loss = 0.05, ..., intercept = 0, slope  = 1, transf = FALSE, x){
+    if(!missing(x)){
+        p_loss <- x
+        warn_about_x("VaR")
+    }
+
+    lst <- apply(dist, 2, identity, simplify = FALSE) # to use in 'mapply'
+
+    res <- mapply(VaR.numeric, lst, intercept = intercept, slope  = slope, transf = transf,
+                  MoreArgs = list(p_loss = p_loss, ...))
+    res
 }
 
 
 
-## VaR <- function(dist, p_loss = 0.05, dist.type = "qf", ...,
-##                 intercept = 0, slope  = 1, tol = .Machine$double.eps^0.5){
-##     dist <- match.fun(dist)
-##     ## TODO: add "rand" to dist.type
-##     ## TODO: include 'dist' in the condition and mapply()?
-##     if(length(p_loss) == 1 && all(sapply(list(...), length) < 2))
-##         res <- switch(dist.type,
-##                       qf = - dist(p_loss, ...),
-##                       cdf = - cdf2quantile(p_loss, dist, tol = tol, ...), # TODO: interval?
-##                       pdf = {
-##                           stop("Not ready yet. Please supply quantile function or cdf")
-##                       },
-##                       stop('argument dist.type should be one of "qf", "cdf" or "pdf"')
-##                       )
-##     else{
-##             # fu <- switch(dist.type,
-##             #              qf = function(y) - dist(y, ...),
-##             #              cdf = function(y) - cdf2quantile(y, dist, tol = tol, ...),
-##             #              pdf = {
-##             #                  stop("Not ready yet. Please supply quantile function or cdf")
-##             #              },
-##             #              stop('argument dist.type should be one of "qf", "cdf" or "pdf"')
-##             #              )
-##             # sapply(p_loss, fu)
-##         res <- - switch(dist.type,
-##                         qf = mapply(dist, p_loss, ...),
-##                         cdf = mapply(cdf2quantile, p_loss, ..., MoreArgs = list(cdf = dist, tol = tol)),
-##                         pdf = {
-##                             stop("Not ready yet. Please supply quantile function or cdf")
-##                         },
-##                         stop('argument dist.type should be one of "qf", "cdf" or "pdf"')
-##                         )
-##
-##     }
-##     - intercept + slope * res
-## }
-
-#' @title Compute expected shortfall (ES) of distributions
+#' @title Compute expected shortfall (ES)
 #'
 #' @description
-#'     Compute the expected shortfall for a distribution. 
+#'     Compute expected shortfall (ES).
 #'
-#' @details \code{ES} computes the expected shortfall for distributions specified by the
-#'     arguments. \code{dist} is typically a function (or the name of one). What \code{dist}
-#'     computes is determined by \code{dist.type}, whose default setting is \code{"qf"} (the
-#'     quantile function). Other possible settings of \code{dist.type} include \code{"cdf"}
-#'     and \code{"pdf"}.  Additional arguments for \code{dist} can be given with the
-#'     \code{"..."} arguments.
+#' @details
+#' 
+#'     \code{ES} is a generic function for computation of expected shortfall. The default
+#'     method computes ES corresponding to a distribution, usually fitted or implied by a
+#'     fitted model.  There are also methods for data (numeric or matrix).
+#'
+#'     Alternative terms for ES include CVaR (conditional value at risk), AVaR (average value
+#'     at risk), and ETL (expected tail loss).
+#' 
+#'     The default method of \code{ES} computes the expected shortfall for distributions
+#'     specified by the arguments. \code{dist} is typically a function (or the name of
+#'     one). What \code{dist} computes is determined by \code{dist.type}, whose default
+#'     setting is \code{"qf"} (the quantile function). Other possible settings of
+#'     \code{dist.type} include \code{"cdf"} and \code{"pdf"}.  Additional arguments for
+#'     \code{dist} can be given with the \code{"..."} arguments.
 #'
 #'     Argument \code{dist} can also be a numeric vector. In that case the ES is computed,
 #'     effectively, for the empirical cumulative distribution function (ecdf) of the
@@ -276,6 +286,8 @@ VaR.numeric <- function(dist, p_loss = 0.05, ..., intercept = 0, slope  = 1, x){
 #'     passed eventually to \code{quantile()} and can be used, for example, to select a
 #'     non-defult method for the computation of quantiles.
 #'
+#'     If \code{dist} is a matrix, the numeric method is applied to each of its columns.
+#'
 #'     Except for the exceptions discussed below, a function computing VaR for the specified
 #'     distribution is constructed and the expected shortfall is computed by numerically
 #'     integrating it. The numerical integration can be fine-tuned with argument
@@ -289,7 +301,7 @@ VaR.numeric <- function(dist, p_loss = 0.05, ..., intercept = 0, slope  = 1, x){
 #'     comparison purposes.
 #'
 #'
-#'     A bunch of expected shortfalls is computed if argument \code{x} or any of the
+#'     A bunch of expected shortfalls is computed if argument \code{p_loss} or any of the
 #'     arguments in \code{"..."} are of length greater than one. They are recycled to equal
 #'     length, if necessary, using the normal \R recycling rules.
 #'
@@ -311,10 +323,13 @@ VaR.numeric <- function(dist, p_loss = 0.05, ..., intercept = 0, slope  = 1, x){
 #'     were chosen for their expressiveness and to minimise the possibility for a clash with
 #'     parameters of \code{dist} (e.g., the Gamma distribution has parameter \code{scale}).
 #'
+#'     When argument \code{dist} represents log-returns, ES is for the log-returns. Use
+#'     \code{transf = TRUE} to return its value for the returns. Note that the ES of the
+#'     returns cannot be obtained by exponentiating the ES for the log-returns.
 #'
 #' @param dist specifies the distribution whose ES is computed, usually a function or a name
-#'     of a function computing quantiles, cdf, pdf, or a random number generator, see
-#'     Details.
+#'     of a function computing quantiles, cdf, pdf, or a random number generator. Can also be
+#'     a numeric vector or matrix, representing data, see section \sQuote{Details}.
 #'
 #' @param p_loss level, default is 0.05.
 #'
@@ -328,18 +343,29 @@ VaR.numeric <- function(dist, p_loss = 0.05, ..., intercept = 0, slope  = 1, x){
 #'
 #' @param control additional control parameters for the numerical integration routine.
 #'
+#' @param transf use only if \code{"dist"} represents log-returns. If \code{TRUE}, give the
+#'     result for the returns (not log-returns), see section \sQuote{Details}.
+#' 
 #' @param ... passed on to \code{dist}.
 #'
 #' @param x deprecated and will soon be removed. \code{x} was renamed to \code{p_loss},
 #'     please use the latter.
 #'
 #'
-#' @return a numeric vector
+#' @return a numeric vector or matrix
 #'
 #' @seealso \code{\link{VaR}} for VaR,
 #'
 #'     \code{\link[=predict.garch1c1]{predict}} for examples with fitted models
 #' 
+#' @concept CVaR
+#' @concept AVaR
+#' @concept ETL
+#' @concept expected shortfall
+#' @concept conditional value at risk
+#' @concept average value at risk
+#' @concept expected tail loss
+#'
 #' @examples
 #' ES(qnorm)
 #'
@@ -390,64 +416,129 @@ CVaR <- AVaR <- ES # synonyms for ES, currently not exported
 #'
 #' @export
 ES.default <- function(dist, p_loss = 0.05, dist.type = "qf", qf, ...,
-                       intercept = 0, slope = 1, control = list(), x){
-    if(!missing(x)){
-        p_loss <- x
-        warn_about_x("ES")
-    }
-
-    dist <- match.fun(dist)
-
-    if(length(p_loss) == 1 && all(sapply(list(...), length) < 2)){
-        if(dist.type == "pdf"){
-            fu <- function(p) - p * dist(p, ...) / p_loss
-
-            iargs <- list(f = fu, lower = -Inf, upper = qf(p_loss)) # stop.on.error = FALSE,?
-
-        }else{
-            fu <- function(p) VaR(dist = dist, p_loss = p, dist.type = dist.type, ...) / p_loss
-
-            iargs <- list(f = fu, lower = 0, upper = p_loss) # stop.on.error = FALSE,?
-        }
-
-        iargs[names(control)] <- control
-
-        res <- do.call("integrate", iargs)$value
-    }else{
-        ## TODO: this is lazy
-        if(dist.type == "pdf"){
-            if(is.list(qf))
-                res <- mapply(ES, MoreArgs = list(dist = dist, dist.type = dist.type, control = control),
-                              p_loss = p_loss, qf = qf, ...)
-            else
-                res <- mapply(ES, MoreArgs = list(dist = dist, dist.type = dist.type, control = control, qf = qf),
-                          p_loss = p_loss, ...)
-                
-        }else{
-            res <- mapply(ES, MoreArgs = list(dist = dist, dist.type = dist.type, control = control),
-                          p_loss = p_loss, ...)
-        }
-    }
-
-    -intercept + slope * res
-}
-
-#' @name ES
-#'
-#' @export
-ES.numeric <- function(dist, p_loss = 0.05, dist.type = "qf", qf, ...,
-                       intercept = 0, slope = 1, control = list(), x){
+                       intercept = 0, slope = 1, control = list(), transf = FALSE, x){
     if(!missing(x)){
         p_loss <- x
         warn_about_x("ES")
     }
     
+    ## TODO: this method needs consolidation; it became repetitive when features were added.
+    
+    dist <- match.fun(dist)
+
+    stopifnot(length(transf) == 1) # only scalar transf for this method
+    if(transf){
+        ## can't apply intercept and slope at the end here;
+        ## so pass them in each call to VaR or (recursively) ES
+        if(length(p_loss) == 1 && all(sapply(list(..., intercept, slope), length) < 2)){
+            if(dist.type == "pdf"){
+                fu <- function(p) - (exp(-intercept + slope * p) - 1) * dist(p, ...) / p_loss
+        
+                iargs <- list(f = fu, lower = -Inf, upper = qf(p_loss)) # stop.on.error = FALSE,?
+        
+            }else{
+                ## TODO: check the theoretical justification here
+                fu <- function(p){
+                    v <- VaR(dist = dist, p_loss = p, dist.type = dist.type, intercept = intercept, slope = slope, ...)
+                    - (exp(-v) - 1)  / p_loss
+                }
+        
+                iargs <- list(f = fu, lower = 0, upper = p_loss) # stop.on.error = FALSE,?
+            }
+        
+            iargs[names(control)] <- control
+        
+            res <- do.call("integrate", iargs)$value
+        }else{
+            if(dist.type == "pdf"){
+                if(is.list(qf))
+                    res <- mapply(ES, MoreArgs = list(dist = dist, dist.type = dist.type, control = control, transf = transf, intercept = intercept, slope = slope),
+                                  p_loss = p_loss, qf = qf, ...)
+                else
+                    res <- mapply(ES, MoreArgs = list(dist = dist, dist.type = dist.type, control = control, transf = transf, intercept = intercept, slope = slope, qf = qf),
+                              p_loss = p_loss, ...)
+                    
+            }else{
+                res <- mapply(ES, MoreArgs = list(dist = dist, dist.type = dist.type, control = control), intercept = intercept, slope = slope, transf = transf,
+                              p_loss = p_loss, ...)
+            }
+        }
+        
+        res
+    }else{
+        if(length(p_loss) == 1 && all(sapply(list(...), length) < 2)){
+            if(dist.type == "pdf"){
+                fu <- function(p) - p * dist(p, ...) / p_loss
+        
+                iargs <- list(f = fu, lower = -Inf, upper = qf(p_loss)) # stop.on.error = FALSE,?
+        
+            }else{
+                fu <- function(p) VaR(dist = dist, p_loss = p, dist.type = dist.type, ...) / p_loss
+        
+                iargs <- list(f = fu, lower = 0, upper = p_loss) # stop.on.error = FALSE,?
+            }
+        
+            iargs[names(control)] <- control
+        
+            res <- do.call("integrate", iargs)$value
+        }else{
+            if(dist.type == "pdf"){
+                if(is.list(qf))
+                    res <- mapply(ES, MoreArgs = list(dist = dist, dist.type = dist.type, control = control),
+                                  p_loss = p_loss, qf = qf, ...)
+                else
+                    res <- mapply(ES, MoreArgs = list(dist = dist, dist.type = dist.type, control = control, qf = qf),
+                              p_loss = p_loss, ...)
+                    
+            }else{
+                res <- mapply(ES, MoreArgs = list(dist = dist, dist.type = dist.type, control = control),
+                              p_loss = p_loss, ...)
+            }
+        }
+        
+        -intercept + slope * res
+    }
+}
+
+#' @name ES
+#'
+#' @export
+ES.numeric <- function(dist, p_loss = 0.05, ..., intercept = 0, slope = 1, transf = FALSE, x){
+    if(!missing(x)){
+        p_loss <- x
+        warn_about_x("ES")
+    }
+
     v <- VaR.numeric(dist, p_loss = p_loss, ..., intercept = intercept, slope = slope)
-    bad <- dist[dist <= - v]
-    res <- - mean(bad)
+    f <- if(transf){
+             function(x){
+                 logret <- dist[dist <= -x]
+                 - mean(exp(logret) - 1)
+             }
+         }else                          # direct code doesn't work if length(v) > 1):
+                                        #     bad <- dist[dist <= - v]
+                                        #     res <- - mean(bad)
+             function(x) - mean(dist[dist <= -x])
+#browser()             
+    res <- sapply(v, f)
 
     res
 }
 
+#' @name ES
+#'
+#' @export
+ES.matrix <- function(dist, p_loss = 0.05, ..., intercept = 0, slope = 1, transf = FALSE, x){
+    if(!missing(x)){
+        p_loss <- x
+        warn_about_x("ES")
+    }
+    
+    ## v <- VaR.matrix(dist, p_loss = p_loss, ..., intercept = intercept, slope = slope)
 
+    lst <- apply(dist, 2, identity, simplify = FALSE) # to use in 'mapply'
 
+    res <- mapply(ES.numeric, lst, intercept = intercept, slope  = slope, transf = transf,
+                  MoreArgs = list(p_loss = p_loss, ...))
+    res
+}
